"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  loadShikiTheme: () => loadShikiTheme,
  pluginShiki: () => pluginShiki
});
module.exports = __toCommonJS(src_exports);
var import_core = require("@expressive-code/core");

// src/cache.ts
var import_shiki = require("shiki");
var highlighterPromise;
async function getCachedHighlighter({ theme }) {
  if (highlighterPromise === void 0) {
    highlighterPromise = (0, import_shiki.getHighlighter)({ theme });
  }
  const highlighter = await highlighterPromise;
  if (!highlighter.getLoadedThemes().includes(theme.name)) {
    await highlighter.loadTheme(theme);
  }
  return highlighter;
}

// src/index.ts
var import_shiki2 = require("shiki");
async function loadShikiTheme(bundledThemeName) {
  const shikiTheme = await (0, import_shiki2.loadTheme)(import_shiki2.BUNDLED_THEMES.includes(bundledThemeName) ? `themes/${bundledThemeName}.json` : bundledThemeName);
  const shikiThemeWithoutType = { ...shikiTheme };
  delete shikiThemeWithoutType.type;
  return new import_core.ExpressiveCodeTheme(shikiThemeWithoutType);
}
function pluginShiki() {
  return {
    name: "Shiki",
    hooks: {
      performSyntaxAnalysis: async ({ codeBlock, theme }) => {
        const highlighter = await getCachedHighlighter({ theme });
        const codeLines = codeBlock.getLines();
        let code = codeBlock.code;
        if (isTerminalLanguage(codeBlock.language)) {
          code = code.replace(/<([^>]*[^>\s])>/g, "X$1X");
        }
        const tokenLines = codeBlock.language === "ansi" ? highlighter.ansiToThemedTokens(code, theme.name) : highlighter.codeToThemedTokens(code, codeBlock.language, theme.name, { includeExplanation: false });
        tokenLines.forEach((line, lineIndex) => {
          if (codeBlock.language === "ansi")
            removeAnsiSequencesFromCodeLine(codeLines[lineIndex], line);
          let charIndex = 0;
          line.forEach((token) => {
            const tokenLength = token.content.length;
            const tokenEndIndex = charIndex + tokenLength;
            const fontStyle = token.fontStyle || import_shiki2.FontStyle.None;
            codeLines[lineIndex].addAnnotation(
              new import_core.InlineStyleAnnotation({
                color: token.color || theme.fg,
                italic: (fontStyle & import_shiki2.FontStyle.Italic) === import_shiki2.FontStyle.Italic,
                bold: (fontStyle & import_shiki2.FontStyle.Bold) === import_shiki2.FontStyle.Bold,
                underline: (fontStyle & import_shiki2.FontStyle.Underline) === import_shiki2.FontStyle.Underline,
                inlineRange: {
                  columnStart: charIndex,
                  columnEnd: tokenEndIndex
                }
              })
            );
            charIndex = tokenEndIndex;
          });
        });
      }
    }
  };
}
function isTerminalLanguage(language) {
  return ["shellscript", "shell", "bash", "sh", "zsh"].includes(language);
}
function removeAnsiSequencesFromCodeLine(codeLine, lineTokens) {
  const newLine = lineTokens.map((token) => token.content).join("");
  const rangesToRemove = getRemovedRanges(codeLine.text, newLine);
  for (let index = rangesToRemove.length - 1; index >= 0; index--) {
    const [start, end] = rangesToRemove[index];
    codeLine.editText(start, end, "");
  }
}
function getRemovedRanges(original, edited) {
  const ranges = [];
  let from = -1;
  let orgIdx = 0;
  let edtIdx = 0;
  while (orgIdx < original.length && edtIdx < edited.length) {
    if (original[orgIdx] !== edited[edtIdx]) {
      if (from === -1)
        from = orgIdx;
      orgIdx++;
    } else {
      if (from > -1) {
        ranges.push([from, orgIdx]);
        from = -1;
      }
      orgIdx++;
      edtIdx++;
    }
  }
  if (edtIdx < edited.length)
    throw new Error(`Edited string contains characters not present in original (${JSON.stringify({ original, edited })})`);
  if (orgIdx < original.length)
    ranges.push([orgIdx, original.length]);
  return ranges;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  loadShikiTheme,
  pluginShiki
});
//# sourceMappingURL=index.cjs.map